clc; 
clear;
close all

load('P0Variables')

%% Select relevant feedback loops using gains in Simulink:
case1 = 1;
z1feed = Pnom;
z2feed = q2nom;
z3feed = q3nom;
z4feed = q4nom;

%% Controller parameters - tunings from 01
%PI Controller for pressure (Same as before)
controller_P_P  = 0;
controller_P_I  = 0;

%Outlet Controllers
controller_q2_I = 0; %k=1, theta(delay) = 0, tau_c = 1
controller_q3_I = 0; %k=1, theta(delay) = 0, tau_c = 1
controller_q4_I = 0; %k=1, theta(delay) = 0, tau_c = 1

%Anti Windup Parameters
Kb   = 0;
Kbq2 = 0;
Kbq3 = 0;
Kbq4 = 0;

%% Simulate step in z1 to tune PI controller for P control
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = z1nom*0.1;                            % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

%% Plot pressures and and flows
%pressures
plotname = 'Open loop response, Step on z_1';
number   = 1;
save('P0Workspace');
P0Plot();

%% Tune PI controller for P control
tauC = V/((qnom*R*T)/Pnom);

k_p     = 0.51/0.05;      %10
tau_p   = 10000;           %57
theta_p = 0;            %0
tau_p_c = tauC;        %25    %1
tau_p_I = min(tau_p,4*tau_p_c);    %4

controller_P_P  = 1/k_p * tau_p/(theta_p+tau_p_c);
controller_P_I  = controller_P_P/tau_p_I;

%Outlet Controllers
controller_q2_I = 0; %k=1, theta(delay) = 0, tau_c = 1
controller_q3_I = 0; %k=1, theta(delay) = 0, tau_c = 1
controller_q4_I = 0; %k=1, theta(delay) = 0, tau_c = 1

%Anti Windup Parameters
Kb   = controller_P_I;
Kbq2 = 0;
Kbq3 = 0;
Kbq4 = 0;

%% Simulate SP change for P, to see PI controller response
z1st   = Pnom*0.1;                             % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'PI controller for P tuned, +10% SP change for P';
number   = 2;
save('P0Workspace');
P0Plot();

%% Simulate step in z2 to tune I controller for q2
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = z2nom*0.1;                            % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Open loop response, Step on z_2';
number   = 3;
save('P0Workspace');
P0Plot();

%% Tune PI controller for P control
tauC = V/((qnom*R*T)/Pnom);

k_p     = 0.51/0.05;      %10
tau_p   = 10000;           %57
theta_p = 0;            %0
tau_p_c = tauC;        %25    %1
tau_p_I = min(tau_p,4*tau_p_c);    %4

controller_P_P  = 1/k_p * tau_p/(theta_p+tau_p_c);
controller_P_I  = controller_P_P/tau_p_I;

%Outlet Controllers
k2     = 0.05/0.05;
theta2 = 0;
controller_q2_I = 1/k2 * 1/(theta2+tau_p_c); %k=1, theta(delay) = 0, tau_c = 1
controller_q3_I = 0; %k=1, theta(delay) = 0, tau_c = 1
controller_q4_I = 0; %k=1, theta(delay) = 0, tau_c = 1

%Anti Windup Parameters
Kb   = controller_P_I;
Kbq2 = controller_q2_I;
Kbq3 = 0;
Kbq4 = 0;

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = q2nom*0.1;                               % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'I controller for q2 tuned, +10% SP change for q2';
number   = 4;
save('P0Workspace');
P0Plot();


%% Simulate step in z2 to tune I controller for q2
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                            % Valve one input
z3dist = z3nom*0.1;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Open loop response, Step on z_3';
number   = 5;
save('P0Workspace');
P0Plot();

%% Tune PI controller for P control
tauC = V/((qnom*R*T)/Pnom);

k_p     = 0.51/0.05;      %10
tau_p   = 10000;           %57
theta_p = 0;            %0
tau_p_c = tauC;        %25    %1
tau_p_I = min(tau_p,4*tau_p_c);    %4

controller_P_P  = 1/k_p * tau_p/(theta_p+tau_p_c);
controller_P_I  = controller_P_P/tau_p_I;

%Outlet Controllers
k2     = 0.05/0.05;
theta2 = 0;
k3     = 0.075/0.05;
theta3 = 0;
controller_q2_I = 1/k2 * 1/(theta2+tau_p_c); 
controller_q3_I = 1/k3 * 1/(theta3+tau_p_c); 
controller_q4_I = 0; %k=1, theta(delay) = 0, tau_c = 1

%Anti Windup Parameters
Kb   = controller_P_I;
Kbq2 = controller_q2_I;
Kbq3 = controller_q3_I;
Kbq4 = 0;

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                               % q2 SP input
z3st   = q3nom*0.1;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'I controller for q3 tuned, +10% SP change for q3';
number   = 6;
save('P0Workspace');
P0Plot();

%% Simulate step in z2 to tune I controller for q2
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                            % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = z4nom*0.1;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Open loop response, Step on z_3';
number   = 7;
save('P0Workspace');
P0Plot();

%% Tunings all controllers tuned. 
tauC = V/((qnom*R*T)/Pnom);

k_p     = 0.51/0.05;      %10
tau_p   = 10000;           %57
theta_p = 0;            %0
tau_p_c = tauC;        %25    %1
tau_p_I = min(tau_p,4*tau_p_c);    %4

controller_P_P  = 1/k_p * tau_p/(theta_p+tau_p_c);
controller_P_I  = controller_P_P/tau_p_I;

%Outlet Controllers
k2     = 0.05/0.05;
theta2 = 0;
k3     = 0.075/0.05;
theta3 = 0;
k4     = 0.075/0.05;
theta4 = 0;
controller_q2_I = 1/k2 * 1/(theta2+tau_p_c); 
controller_q3_I = 1/k3 * 1/(theta3+tau_p_c); 
controller_q4_I = 1/k4 * 1/(theta4+tau_p_c); 

%Anti Windup Parameters
Kb   = controller_P_I;
Kbq2 = controller_q2_I;
Kbq3 = controller_q3_I;
Kbq4 = controller_q4_I;

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                               % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = q4nom*0.1;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'I controller for q4 tuned, +10% SP change for q4';
number   = 8;
save('P0Workspace');
P0Plot();

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = P1;                                   % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Closed Loop Responses PI-I-I-I, +100% Dist on P1';
number   = 9;
save('P0Workspace');
P0Plot();

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = P2;                                   % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Closed Loop Responses PI-I-I-I, +100% Dist on P2';
number   = 10;
save('P0Workspace');
P0Plot();

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = P3;                                   % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Closed Loop Responses PI-I-I-I, +100% Dist on P3';
number   = 11;
save('P0Workspace');
P0Plot();

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = P4;                                   % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Closed Loop Responses PI-I-I-I, +100% Dist on P4';
number   = 12;
save('P0Workspace');
P0Plot();

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = -P1*0.25;                              % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Closed Loop Responses PI-I-I-I, -25% Dist on P1';
number   = 13;
save('P0Workspace');
P0Plot();

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = -P2*0.25;                              % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Closed Loop Responses PI-I-I-I, -25% Dist on P2';
number   = 14;
save('P0Workspace');
P0Plot();

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = -P3*0.25;                              % Disturbance P3 input
P4dist = 0;                                    % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Closed Loop Responses PI-I-I-I, -25% Dist on P3';
number   = 15;
save('P0Workspace');
P0Plot();

%% Simulate SP change for P, to see PI controller response
z1st   = 0;                                    % Pressure SP input
z2st   = 0;                                    % q2 SP input
z3st   = 0;                                    % q3 SP input
z4st   = 0;                                    % q4 SP input
z1dist = 0;                                    % Valve one input
z2dist = 0;                                    % Valve one input
z3dist = 0;                                    % Valve one input
z4dist = 0;                                    % Valve one input
P1dist = 0;                                    % Disturbance P1 input
P2dist = 0;                                    % Disturbance P2 input
P3dist = 0;                                    % Disturbance P3 input
P4dist = -P4*0.25;                              % Disturbance P4 input

sim('P0PressureFlowODESim')                    % Run Simulink file

qnomplot = t.^(0)*2.2;                         % Prepare plotting variables
q2plot   = q2stepplot;                         % Prepare plotting variables
q3plot   = q3stepplot;                         % Prepare plotting variables
q4plot   = q4stepplot;                         % Prepare plotting variables

save(fullfile(savdir,'P0201CL'));              % Save variables for later

%% Plot pressures and and flows
%pressures
plotname = 'Closed Loop Responses PI-I-I-I, -25% Dist on P4';
number   = 16;
save('P0Workspace');
P0Plot();

