% Plotting options
set(0,'DefaultTextFontName','Times',...
'DefaultTextFontSize',18,...
'DefaultAxesFontName','Times',...
'DefaultAxesFontSize',18,...
'DefaultLineLineWidth',2,...
'DefaultLineMarkerSize',7.75,...
'DefaultStairLineWidth',2)

%savdir = '/Users/erikandrevik/Dropbox/MATLAB/Project2020/Erik2/88 Data from Simulations';
% For saving files, uncomment and use a savdir as the one above for your
% pc
%% Small pressures. small flow. 

%% Parameters

%load FixedValve.mat    % small pressures
%load Turbine.mat    % large pressures

%Initial Pressures
P1    = 15;                                     % [bar], reservoir pressure
Pnom = 10;                         % [bar], nominal pipe pressure after pump
P2 = 2;                                     % [bar], nominal pipe2 pressure          
P3 = 2;                                     % [bar], nominal pipe3 pressure
P4 = 2;

qnom  = 2.0;                                                 % [kmol/s], flow
q2nom = 0.5; %Instead we use anom right now to split streams equally
q3nom = 0.75; %Instead we use anom right now to split streams equally
q4nom = 0.75; %Instead we use anom right now to split streams equally
%These are used now. for nominal

z1nom = 0.5;                    % [-], nominal valve opening
z2nom = 0.5;                    % [-], nominal valve opening
z3nom = 0.5;                    % [-], nominal valve opening
z4nom = 0.5;                    % [-], nominal valve opening

V     = 150;                                              % [m3] pipe volume
T     = 200+273;                                 % [K] constant temperature
R     = 8.314*1e-5;                           % [bar*m3/kmol/K] gas constant

anom=0.5;                          %[-] nominal split factor to change qnom

Cv1 = qnom/z1nom/sqrt(P1^2-Pnom^2);                             % Large valve coeff
Cv2 = 2/8*qnom/z2nom/sqrt(Pnom^2-P2^2)  ;%Different qnom for cv2 and cv3 bc split                        
Cv3 = 3/8*qnom/z3nom/sqrt(Pnom^2-P3^2);                   % fixed valve coeff
Cv4 = 3/8*qnom/z4nom/sqrt(Pnom^2-P4^2);                   % fixed valve coeff

param = [Cv1;
    Cv2;
    Cv3;
    Cv4;
    V;
    R;
    T];

% For simulation - Dimensions 
nx = 1; 
nu =4; 
np = size(param,1); 
nz = 4;

Kdiff = [eye(nx) zeros(nx,nz)];             % selection matrix for diff var
Kalg  = [zeros(nz,nx) eye(nz)];              % selection matrix for alg var
tsim  = 400000;                                          % [s] simulation time
x0    = [Pnom];                                 % [bar], initial conditions
%x0     = [1, 1, 0.5];
tstep = 100;                                                 % [s] step time

%Disturbances
disturbP1 = P1;
disturbP2 = P2;
disturbP3 = P3; 
disturbP4 = P4;

%% Open loop controller parameters

%% Controller parameters - tunings from 01
%PI Controller for pressure (Same as before)
controller_P_P  = 0;
controller_P_I  = 0;

%Outlet Controllers
controller_q2_P = 0;
controller_q3_P = 0;
controller_q4_P = 0;
controller_q2_I = 0; %k=1, theta(delay) = 0, tau_c = 1
controller_q3_I = 0; %k=1, theta(delay) = 0, tau_c = 1
controller_q4_I = 0; %k=1, theta(delay) = 0, tau_c = 1
controller_q3_Pdroop = 1;
controller_q3_Idroop = 0;
controller_q4_Pdroop = 1;
controller_q4_Idroop = 0;

%Anti Windup Parameters
Kb   = 0;
Kbq2 = 0;
Kbq3 = 0;
Kbq4 = 0;

%Droop loop on/off - 0 = off, 1 = on
droop1 = 0;
droop2 = 0;

%I control loop on/off - 0 = off, 1 = on
case1 = 0;

%P control loop on/off - 0 = off, 1 = on
case2 = 0;

%Nominal feed values, need to be given values in script
z1feed = 0;
z2feed = 0;
z3feed = 0;
z4feed = 0;

%Droop controllers step values - 0 as standard
droop1st = 0;
droop2st = 0;
%% Save to workspace
save('P0Variables')